use btproto::protocol;
use serde::{Serialize, Deserialize};

use crate::{sector_proto::FileId, CallMsg, ActorName};

#[derive(Serialize, Deserialize)]
pub struct Open {
    id: FileId
}

impl CallMsg for Open {
    type Reply = ActorName;
}

protocol! {
    let name = FsProtocol;
    let states = [
        ServerInit, Listening,
        Client,
        FileInit, Open, Opened,
        FileHandle,
    ];
    ServerInit?Activate -> Listening;

    Client -> Client, service(Listening)!Query;
    Listening?Query -> Listening, Client!Query::Reply;
    Client?Query::Reply -> Client;

    Client -> Client, service(Listening)!Open;
    Listening?Open -> Listening, Opened, Client!Open::Reply[Opened];
    Client?Open::Reply[Opened] -> Client, FileHandle[Opened];

    FileInit?Activate -> FileInit;
    FileInit?Open -> Opened;

    FileHandle[Opened] -> FileHandle[Opened], Opened!FileOp;
    Opened?FileOp -> Opened, Client!FileOp::Reply;
    FileHandle?FileOp::Reply -> FileClient;

    FileHandle[Opened] -> End, Opened!Close;
    Opened?Close -> End;
}